@@ -9,7 +9,7 @@ use Sys::Linux::Unshare qw/:all/;
 use POSIX qw/_exit/;
 
 use Moo;
-use Carp qw/carp/;
+use Carp qw/croak/;
 
 has no_proc => (is => 'rw');
 
@@ -22,7 +22,7 @@ sub _uflags {
   my $self = shift;
   my $uflags = 0;
 
-  $uflags |= CLONE_NEWNS if ($self->private_tmp || $self->private_mount);
+  $uflags |= CLONE_NEWNS if ($self->private_tmp || $self->private_mount || ($self->private_pid && !$self->no_proc));
   $uflags |= CLONE_NEWPID if ($self->private_pid);
   $uflags |= CLONE_NEWNET if ($self->private_net);
   $uflags |= CLONE_NEWIPC if ($self->private_ipc);
@@ -35,11 +35,11 @@ sub _uflags {
 
 sub _subprocess {
   my ($self, $code, %args) = @_;
-  die "_subprocess requires a CODE ref" unless ref $code eq 'CODE';
+  croak "_subprocess requires a CODE ref" unless ref $code eq 'CODE';
 
   my $pid = fork();
 
-  carp "Failed to fork: $!" if ($pid < 0);
+  croak "Failed to fork: $!" if ($pid < 0);
   if ($pid) {
     waitpid($pid, 0); # block and wait on child
     return $?;
@@ -52,16 +52,16 @@ sub _subprocess {
 sub pre_setup {
   my ($self, %args) = @_;
 
-  die "Private net is not yet supported" if $self->private_net;
+  croak "Private net is not yet supported" if $self->private_net;
   if ($self->private_pid && (ref $args{code} ne 'CODE' || !$args{_run})) {
-    die "Private PID space requires a coderef to become the new PID 1";
+    croak "Private PID space requires a coderef to become the new PID 1";
   }
 }
 
 sub post_setup {
   my ($self, %args) = @_;
   # If we want a private /tmp, or private mount we need to recursively make every mount private.  it CAN be done without that but this is more reliable.
-  if ($self->private_mount || $self->private_tmp) {
+  if ($self->private_tmp || $self->private_mount || ($self->private_pid && !$self->no_proc)) {
       mount("/", "/", undef, MS_REC|MS_PRIVATE, undef);
   }
 
@@ -69,15 +69,13 @@ sub post_setup {
     my $data = undef;
     $data = $self->private_tmp if (ref $self->private_tmp eq 'HASH');
 
-    eval {umount("/tmp")}; # ignore it if it wasn't mounted
-    mount("/tmp", "/tmp", "tmpfs", 0, undef);
-    mount("/tmp", "/tmp", "tmpfs", MS_PRIVATE, $data);
+    mount("none", "/tmp", "tmpfs", MS_MGC_VAL, undef);
+    mount("none", "/tmp", "tmpfs", MS_PRIVATE, $data);
   }
 
   if ($self->private_pid && !$self->no_proc) {
-    eval {umount("/proc")}; # ignore it if it wasn't mounted already
-    mount("/proc", "/proc", "proc", 0, undef);
-    mount("/proc", "/proc", "proc", MS_PRIVATE, undef);
+    mount("proc", "/proc", "proc", MS_MGC_VAL, undef);
+    mount("proc", "/proc", "proc", MS_PRIVATE|MS_REC, undef);
   }
 }
 
@@ -99,19 +97,21 @@ sub run {
   my $code = $args{code};
   $args{_run} = 1;
 
-  carp "Run must be given a codref to run" unless ref $code eq "CODE";
-  my $uflags = $self->_uflags;
-  $self->pre_setup(%args);
+  croak "Run must be given a codref to run" unless ref $code eq "CODE";
 
   $self->_subprocess(sub {
-    $self->setup(%args);
+    $self->pre_setup(%args);
+    my $uflags = $self->_uflags;
+    unshare($uflags);
 
     # We've just unshared, if we wanted a private pid space we MUST fork again.
     if ($self->private_pid) {
       $self->_subprocess(sub {
+        $self->post_setup(%args);
         $code->(%args);
       }, %args);
     } else {
+      $self->setup(%args);
       $code->(%args);
     }
   }, %args);
